#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <math.h>
#include <time.h>
#include <string.h>

#include "structures.h"

void solver::new_orig_clause(const vector<int> &literals)
{
  clause cur(literals.size());
  cur.id = -(original_clauses.size() + 1);
  
  orig_lits += literals.size();
  for (size_t i = 0 ; i < literals.size() ; ++i) 
    cur[i] = int_to_lit(literals[i]);
  original_clauses.push_back(cur);
}

// adjust the number of variables in a theory.  Deal with watched lits
// and with heaps for picking a variable to branch on or unit propagate
void solver::resize(size_t n)
{
  variables.resize(1 + n);
  units = heap(variables);
  var_order_heap = heap(variables);
  watched.resize(2 * n);
}

// solver initialization
solver::solver(const vector<set<UPTestingLit> >& clauses, bool &ok) :
  score_inc(1), clause_score_inc(1), num_conflicts_for_next_simplify(0), 
  simplify_orig_kb(false), simplify_learned_kb(false)
{
  decision_lit.push_back(0);
  ok = true;
  resize(atom_name_map.size());

  for (size_t i = 0 ; i < clauses.size() ; ++i) {
    bool skip_clause = false;
    vector<int> literals;
    for (set<UPTestingLit>::iterator it = clauses[i].begin() ; 
         it != clauses[i].end() ; ++it)
      literals.push_back(it->int_lit());

    if (clauses[i].size() == 1) { // singleton clause
      if (status(int_to_atom(literals[0])) == 0)
        enqueue(int_to_lit(literals[0]));
      skip_clause = true;
    }
    else if (clauses[i].empty()) ok = false; // empty clause!

    if (!skip_clause) new_orig_clause(literals);
  }
}

// finish initialization: watched lits, set up initial variable order,
// process any initial unit lits, simplify kb, set maximum number of
// learned clauses to 1/3 original DB size.

bool solver::finish_initialization()
{ 
  initialize_watchers();

  for (size_t i = 1 ; i < variables.size() ; ++i) var_order_heap.insert(i);

  if (process_unit_literal_queue()) return false; // non-zero id means conflict
  
  simplify(false);
  max_learned_clauses = original_clauses.size() / 3;  
  return true;
}
