// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <iostream>
#include "structures.h"

void heap::move_up(int idx)
{
  atom a = (*this)[idx];
  for (int parent = (idx - 1) >> 1 ; parent >= 0 && cmp(a,(*this)[parent]) ; 
       parent = (parent - 1) >> 1) {
    int i = (*this)[parent];
    (*this)[idx] = i;
    location[i] = idx;
    idx = parent;
  }
  (*this)[idx] = a;
  location[a] = idx;
}

void heap::move_down(int idx)
{
  atom a = (*this)[idx];  
  while (2 * idx + 1 < (int) size()) {
    int child = 2 * idx + 1;
    int bigger_child = child +
      (child + 1 < (int) size() && cmp((*this)[child + 1],(*this)[child]));
    if (!cmp((*this)[bigger_child],a)) break;
    (*this)[idx] = (*this)[bigger_child];
    location[(*this)[idx]] = idx;
    idx = bigger_child;
  }
  (*this)[idx] = a;
  location[a] = idx;
}

void heap::clear()
{
  vector<atom>::clear();
  fill(location.begin(),location.end(),-1);
}

/* Add a scored element to a heap */
void heap::insert(atom v)
{  
  location[v] = size();
  push_back(v);
  move_up(location[v]);
}

/* Return the top of the heap */
atom heap::front() 
{
  atom ans = (*this)[0];
  (*this)[0] = back();
  location[(*this)[0]] = 0;     // don't pop yet in case queue is empty
  location[ans] = -1;
  pop_back();
  if (size() > 1) move_down(0);
  return ans;
}

/* Rescale the scores in the heap, and the score increment.  */
void solver::rescale_variable_scores()
{
  for (size_t i = 1; i < variables.size(); i++) score(i) *= (1/MAX_SCORE);
  score_inc *= (1/MAX_SCORE);
}

/* Adjust the score of a variable.  Rescale if needed.  */
void solver::increment_literal_score(literal l)
{
  atom v = get_atom(l);

  if (g_up_queue_pop_heuristic_on && // if off, don't maintain scores
      ((score(v) += score_inc) > MAX_SCORE))
    rescale_variable_scores();

  var_order_heap.update(v);
}

/* Divide scores of all clauses and increment by MAX_CLAUSE_SCORE.  */
void solver::rescale_clause_scores() 
{
  for (size_t i = 0 ; i < learned_scores.size() ; ++i)
    learned_scores[i] /= MAX_CLAUSE_SCORE;
  clause_score_inc /= MAX_CLAUSE_SCORE;
}

/* Increment score of a particular learned clause.  Rescale if needed.  */
void solver::increment_clause_score(clause_id c)
{
  if ((learned_scores[c - 1] += clause_score_inc) > MAX_CLAUSE_SCORE)
    rescale_clause_scores();
}
