// THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// AND ITS AUTHOR AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH
// (JAIR) AND JAIR'S PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL
// WARRANTIES, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND ANY
// WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY AND
// RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR
// NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE
// FOR DAMAGES OF ANY KIND RESULTING FROM ITS USE.  Without limiting
// the generality of the foregoing, neither the author, nor JAIR, nor
// JAIR's publishers and distributors, warrant that the Source Code
// will be error-free, will operate without interruption, or will meet
// the needs of the user.

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <string>

#include "structures.h"

void solver::print_progress_header() const
{
  if (!verbose()) return;
  rprintf("+----+-----------------+------------------+----------------------------------+---------------------------+---------+-----------+\n");
  rprintf("| %-2s | %-15s | %-16s | %-32s | %-25s | %-7s | %-9s |\n","Re","Conflicts","Original","Learned","Decisions"," Time","KB");
  rprintf("| %2s | %7s %7s | %7s %8s | %7s %7s %8s %7s | %7s %10s %6s | %7s | %4s %4s |\n","st","Max","Actual","Clauses","Literals","Max","Clauses","Literals","LPC","Total","Per Sec","C/D","","Red.","Sim.");
  rprintf("+----+-----------------+------------------+----------------------------------+---------------------------+---------+-----------+\n");
}

void solver::print_progress_footer() const
{
  if (!verbose()) return;
  rprintf("+----+-----------------+------------------+----------------------------------+---------------------------+---------+-----------+\n");
}

void solver::print_progress() const
{
  if (!verbose()) return;

  clock_t end_time = clock();  
  double time_used = (end_time - start_time) / (double) CLOCKS_PER_SEC;
  int64 cur_decisions = decisions - previous.decisions;
  int64 cur_conflicts = conflicts - previous.conflicts;
  int cur_reduce_kb = kb_reduction_count - previous.kb_reduction_count;
  int cur_simplify_kb = 
    kb_simplification_count - previous.kb_simplification_count;

  previous = *this;

  start_time = end_time;
  
  //print the message
  rprintf("| %2d | %7"i64d" %7"i64d" | %7d %8"i64d" | %7d %7d %8"i64d
          " %7.1f | %7"i64d" %10.2f %6.3f | %7.3f | %4d %4d |\n",
	  r.number, r.next,
	  conflicts,
	  original_clauses.size(), orig_lits,
	  (int) max_learned_clauses, learned_clauses.size(),
	  learned_lits, ((double )learned_lits / learned_clauses.size()),
	  cur_decisions, (double) cur_decisions / time_used,
	  (double) cur_conflicts / cur_decisions,
	  time_used,
	  cur_reduce_kb, cur_simplify_kb
	  );
}

// print a clause, including status/assignment level of each variable in it
void solver::print_clause(const clause &c) const
{
  if (!c.id)  { printf("(NULL)\n"); return; }
  
  printf("(");
  for (size_t i = 0 ; i < c.size() ; ++i) {
    atom a = get_atom(c[i]);
    printf("%d%s(%d), ",lit_to_int(c[i]),
           status(a)? (status(a) == c[i]? "s" : "r") : "",level(a));
  }
  printf(")\n\n");
}

void solver::print_stats() const
{
  rprintf("CNF stats: (%d vars, %d clauses)\n",
          variables.size() - 1,original_clauses.size());
  rprintf("Decisions: %"i64d"\n",decisions);
  rprintf("Conflicts: %"i64d"\n",conflicts);
}

void solver::print_solution() const
{
  printf("v ");
  for (size_t i = 0 ; i < variables.size() - 1 ; ++i)
    printf("%d ",lit_to_int(assign[i]));
  printf("0\n");
}
